@@ -10,6 +10,8 @@ class Command:
     usage=''
     split=True
     logger=logging.getLogger('Shell')
+    word_completer=None
+    file_completer=None
 
     def __init__(self, shell):
         self.shell = shell
@@ -48,29 +50,29 @@ class Exit(Command):
         exit(0)
 
 class Help(Command):
+    word_completer = {'commands': None, 'variables': None, 'all': None}
     usage='''
     help
 
     Usage:
-        help --commands
-        help --variables
-        help --commands --variables
+        help -h
+        help commands
+        help variables
+        help all
 
     Options:
         -h, --help                              Print this help message.
-        -c, --commands                          List all commands.
-        -v, --variables                         List all variables.
     '''
 
     def action(self):
         if self.args is None:
-            self.args = {'--commands': True, '--variables': False}
-        if self.args['--commands'] is not None:
+            return
+        if self.args['commands'] or self.args['all']:
             print('Commands: ')
             for command in self.shell.commands:
                 print('\t- ' + command)
             print('Run `command-name -h` to get info about each command.')
-        if self.args['--variables'] is not None:
+        if self.args['variables'] or self.args['all']:
             print('Builtin Variables: ')
             for variable in self.shell.builtin_variables:
                 print('\t- ' + variable)
@@ -132,13 +134,27 @@ class Unset(Command):
         if self.args['NAME'][0] != '$':
             self.logger.error('Variables must be prefixed with $.')
             return
-        if self.args['NAME'][1:] in self.shell.builtin_variables:
-            self.logger.error('Cannot unset builtin shell variables.')
-            return
-        if self.args['NAME'][1:] not in self.shell.variables:
-            self.logger.error('Variable does not exist.')
-            return
-        del self.shell.variables[self.args['NAME'][1:]]
+        if self.args['NAME'][0] == '$' and self.args['NAME'][1] != '{':
+            if self.args['NAME'][1:] in self.shell.builtin_variables:
+                self.logger.error('Cannot unset builtin shell variables.')
+                return
+            if self.args['NAME'][1:] not in self.shell.variables:
+                self.logger.error('Variable does not exist.')
+                return
+            del self.shell.variables[self.args['NAME'][1:]]
+            self.shell.completer.deleteVariable(self.args['NAME'][1:])
+        elif self.args['NAME'][0] == '$' and self.args['NAME'][1] == '{':
+            if self.args['NAME'][-1] != '}':
+                self.logger.error('Unbalanced curly brackets.')
+                return
+            if self.args['NAME'][2:-1] in self.shell.builtin_variables:
+                self.logger.error('Cannot unset builtin shell variables.')
+                return
+            if self.args['NAME'][2:-1] not in self.shell.variables:
+                self.logger.error('Variable does not exist.')
+                return
+            del self.shell.variables[self.args['NAME'][2:-1]]
+            self.shell.completer.deleteVariable(self.args['NAME'][2:-1])
 
 class Set(Command):
     split=False
@@ -175,6 +191,7 @@ class Set(Command):
                 self.shell.builtin_variables[name] = value
             else:
                 self.shell.variables[name] = value
+                self.shell.completer.addVariable(name)
         except NameError as e:
             # Already handled inside parseExpression
             pass
@@ -182,6 +199,7 @@ class Set(Command):
             self.logger.error('Couldn\'t parse expression {}.', self.args['EXPR'])
 
 class Source(Command):
+    file_completer = ['sh', 'shell', 'script']
     split=False
     usage='''
     source
